#ifndef SETTINGS_VALUES_H
#define SETTINGS_VALUES_H

/* These values are shown as "unlimited" */
#define SET_VALUE_UNLIMITED "unlimited"
#define SET_UINT_UNLIMITED UINT_MAX
#define SET_SIZE_UNLIMITED UOFF_T_MAX

/* These values are shown as "infinite" */
#define SET_VALUE_INFINITE "infinite"
#define SET_TIME_INFINITE UINT_MAX
#define SET_TIME_MSECS_INFINITE UINT_MAX

#endif
