#ifndef UNICODE_DATA_TABLES_H
#define UNICODE_DATA_TABLES_H

/* This file is automatically generated by unicode-ucd-compile.py from:
     ./ucd/CaseFolding.txt
     ./ucd/CompositionExclusions.txt
     ./ucd/DerivedCoreProperties.txt
     ./ucd/DerivedNormalizationProps.txt
     ./ucd/GraphemeBreakProperty.txt
     ./ucd/PropList.txt
     ./ucd/PropertyValueAliases.txt
     ./ucd/SpecialCasing.txt
     ./ucd/UnicodeData.txt
     ./ucd/WordBreakProperty.txt
     ./ucd/emoji-data.txt
 */

#include "unicode-data-types.h"

#define UNICODE_DECOMPOSITION_MAX_LENGTH 18
#define UNICODE_COMPOSITIONS_MAX_PER_STARTER 19
#define UNICODE_CASE_MAPPING_MAX_LENGTH 3

extern const struct unicode_code_point_data unicode_code_points[];

extern const uint8_t unicode_code_points_index8[];
extern const uint8_t unicode_code_points_index16[];
extern const uint16_t unicode_code_points_index24[];
extern const uint16_t unicode_code_points_index32[];

extern const uint32_t unicode_decompositions[];

extern const uint32_t unicode_compositions[];
extern const uint32_t unicode_composition_primaries[];

extern const uint32_t unicode_case_mappings[];

#endif
