#ifndef IOSTREAM_MULTIPLEX_PRIVATE_H
#define IOSTREAM_MULTIPLEX_PRIVATE_H

#define IOSTREAM_MULTIPLEX_CHANNEL_SWITCH_PREFIX "\x03\xFE"
#define IOSTREAM_MULTIPLEX_CHANNEL_SWITCH_PREFIX_LEN 2

#define IOSTREAM_MULTIPLEX_HEADER_SIZE \
	(7 + IOSTREAM_MULTIPLEX_CHANNEL_SWITCH_PREFIX_LEN)

enum multiplex_istream_switch_type {
	MULTIPLEX_ISTREAM_SWITCH_TYPE_CHANNEL_ID = 0,
	MULTIPLEX_ISTREAM_SWITCH_TYPE_PREFIX_CHAR1 = 1,
};

#endif
