#ifndef UNICODE_DATA_TYPES_H
#define UNICODE_DATA_TYPES_H

/* This file is automatically generated by unicode-ucd-compile.py from:
     ./ucd/CaseFolding.txt
     ./ucd/CompositionExclusions.txt
     ./ucd/DerivedCoreProperties.txt
     ./ucd/DerivedNormalizationProps.txt
     ./ucd/GraphemeBreakProperty.txt
     ./ucd/PropList.txt
     ./ucd/PropertyValueAliases.txt
     ./ucd/SpecialCasing.txt
     ./ucd/UnicodeData.txt
     ./ucd/WordBreakProperty.txt
     ./ucd/emoji-data.txt
 */

#include "unicode-data-static.h"

/* Decomposition_Type */
enum unicode_decomposition_type {
	/* Canonical */
	UNICODE_DECOMPOSITION_TYPE_CANONICAL = 0,
	/* <Compat> */
	UNICODE_DECOMPOSITION_TYPE_COMPAT,
	/* <Circle> */
	UNICODE_DECOMPOSITION_TYPE_CIRCLE,
	/* <Final> */
	UNICODE_DECOMPOSITION_TYPE_FINAL,
	/* <Font> */
	UNICODE_DECOMPOSITION_TYPE_FONT,
	/* <Fraction> */
	UNICODE_DECOMPOSITION_TYPE_FRACTION,
	/* <Initial> */
	UNICODE_DECOMPOSITION_TYPE_INITIAL,
	/* <Isolated> */
	UNICODE_DECOMPOSITION_TYPE_ISOLATED,
	/* <Medial> */
	UNICODE_DECOMPOSITION_TYPE_MEDIAL,
	/* <Narrow> */
	UNICODE_DECOMPOSITION_TYPE_NARROW,
	/* <Nobreak> */
	UNICODE_DECOMPOSITION_TYPE_NOBREAK,
	/* <None> */
	UNICODE_DECOMPOSITION_TYPE_NONE,
	/* <Small> */
	UNICODE_DECOMPOSITION_TYPE_SMALL,
	/* <Square> */
	UNICODE_DECOMPOSITION_TYPE_SQUARE,
	/* <Sub> */
	UNICODE_DECOMPOSITION_TYPE_SUB,
	/* <Super> */
	UNICODE_DECOMPOSITION_TYPE_SUPER,
	/* <Vertical> */
	UNICODE_DECOMPOSITION_TYPE_VERTICAL,
	/* <Wide> */
	UNICODE_DECOMPOSITION_TYPE_WIDE,
};

/* Decomposition_Type */
enum unicode_decomposition_type
unicode_decomposition_type_from_string(const char *str);

#endif
