#ifndef DOVECOT_VERSION_H
#define DOVECOT_VERSION_H

#define DOVECOT_REVISION "e278b1e09b"
#define DOVECOT_VERSION_FULL VERSION" ("DOVECOT_REVISION")"
#define DOVECOT_BUILD_INFO DOVECOT_VERSION_FULL

#endif /* DOVECOT_VERSION_H */
